/*
 * Copyright (c) 2004, Accent .^M
 * All rights reserved. ^M
 *^M
 * Redistribution and use in source and binary forms, with or without ^M
 * modification, are permitted provided that the following conditions ^M
 * are met: ^M
 * 1. Redistributions of source code must retain the above copyright ^M
 *    notice, this list of conditions and the following disclaimer. ^M
 * 2. Redistributions in binary form must reproduce the above copyright ^M
 *    notice, this list of conditions and the following disclaimer in the ^M
 *    documentation and/or other materials provided with the distribution. ^M
 * 3. Neither the name of the Institute nor the names of its contributors ^M
 *    may be used to endorse or promote products derived from this software ^M
 *    without specific prior written permission. ^M
 *^M
 * THIS SOFTWARE IS PROVIDED BY Accent ``AS IS'' AND ^M
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE ^M
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ^M
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE ^M
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL ^M
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS ^M
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ^M
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT ^M
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY ^M
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF ^M
 * SUCH DAMAGE. ^M
 *^M
* This file is part of the lwIP TCP/IP stack.^M
 * ^M
 * Author: Alessandro Timossi <alessandro.timossi@accent.it>^M
 */
#ifndef __BOOTP_H__
#define __BOOTP_H__

#include "lwip/err.h"
#include "lwip/udp.h"
#include "arch/cc.h"
#include "logic.h"
#include "aip_lib.h"


/* TFTP request opcode */
#define REQUEST 0x0100

/* TFTP ACK opcode */
#define ACK 0x0400

/* TFTP request opcode */
#define DATA 0x0300

/* BOOTP server name */
#define BOOTP_SNAME ""

/* BOOTP TFTP file name */
#define BOOTP_FILE       "pippo.txt"
#define BOOTP_FILE_LEN   9

/* BOOTP TFTP file transfer type */
#define BOOTP_MODE       "octet"
#define BOOTP_MODE_LEN   5

/* BOOTP TFTP file option */
#define OPT1             "blksize"
#define OPT1_LEN         7

/* BOOTP TFTP file option value */
#define VALUE1           "12"
#define VALUE1_LEN       2

/* BOOTP global transaction identifier (must be unique for each BOOTP request) */
#define XID_VALUE        0x01

/* BOOTP TFTP pointer to data received after RRQ */
/* Note: LMSSRAMSTART = 0xC1000000 defined in logic.h*/
/*       LMSSRAMEND   = 0xC10FFFFF defined in logic.h*/
#define TFTP_DATA_POINTER     LMSSRAMSTART + DATA_OFFSET
#define DATA_OFFSET           0xFF7FF

/* Max TFTP data file transfer lenght */
#define MAX_TFTP_LEN          512


/* BOOTP message structure */
struct bootp_msg {
  PACKED u8_t  op;
  PACKED u8_t  htype;
  PACKED u8_t  hlen;
  PACKED u8_t  hops;
  PACKED u32_t xid;
  PACKED u16_t secs;
  PACKED u16_t flags;
  PACKED u32_t ciaddr;
  PACKED u32_t yiaddr;
  PACKED u32_t siaddr;
  PACKED u32_t giaddr;
  PACKED u8_t  chaddr[16];
  PACKED u8_t  sname[64];
  PACKED u8_t  file[128];
		PACKED u32_t cookie;
  PACKED u8_t  vend[60];  
};

/* TFTP RRQ (Read Request) message structure */

struct tftp_RRQ_packet {
  PACKED u16_t  opcode;
  PACKED u8_t   filename[BOOTP_FILE_LEN];
  PACKED u8_t   dummy1;
		PACKED u8_t   mode[BOOTP_MODE_LEN];
		PACKED u8_t   dummy2;
		PACKED u8_t   opt1[OPT1_LEN];
		PACKED u8_t   dummy3;
		PACKED u8_t   value1[VALUE1_LEN];
		PACKED u8_t   dummy4;
}; 
 
/* TFTP DATA message structure */
struct tftp_DATA_packet {
  PACKED u16_t  opcode;
  PACKED u16_t  block;
		PACKED u8_t   data[512];
}; 


/* TFTP ACK (Acknowledge) message structure */
struct tftp_ACK_packet {
  PACKED u16_t  opcode;
  PACKED u16_t  block;
}; 



/* BOOTP main structure */
struct bootp
{
  /** transaction identifier of last sent request */ 
  u32_t xid;
  /** our connection to the BOOTP server */ 
  struct udp_pcb *pcb;
		/** our connection to the TFTP server */ 
  struct udp_pcb *tftp_pcb;
		/** End of tftp transfer flag */
		BOOL tftp_end;
		/** tftp data buffer for receiving boot file */
  u8_t   *tftp_data_buffer;
		/** tftp data buffer index */
  int    tftp_data_buffer_index;		
  /** (first) pbuf of incoming msg */
  struct pbuf *p;
  /** incoming msg */
  struct bootp_msg *msg_in;
  /** incoming msg options */
  struct bootp_msg *options_in; 
  /** ingoing msg options length */
  u16_t options_in_len;

  struct pbuf *p_out; /* pbuf of outcoming msg */
  struct bootp_msg *msg_out; /* outgoing msg */
  struct ip_addr server_ip_addr; /* dhcp server address that offered this lease */
  struct ip_addr offered_ip_addr;
  struct ip_addr offered_sn_mask;
  struct ip_addr offered_gw_addr;
  struct ip_addr offered_bc_addr;
};

#endif /* __BOOTP_H__ */
